/*
  - Read values by accessing the member variable
  - Write settings by using the corresponding set* function
    (otherwise, the setting is not stored in persistency
  - TODO: 
      It could be useful to use the application exit handle to 
      write the settings to the persistent cookie. However, the
      settings would not be stored if the application crashes.
      -> think about this tradeoff and decide...
*/

function Settings() {
	// this variable is set to true when the user provided the correct decryption password
	this.decryptionFailed = true;
	// this counter holds the number of failed decryption events 
	// (= attempts of the user with a wrong password) for brute force protection
	this.decryptionFailedCounter = 0;

	// init - default values
	this.init();

	// now read cookie contents and overwrite the defaults
	this.readSettingsCookie();

	// finaly store the settings (merged from default and persistency)
	this.writeSettingsCookie();
}

// the global settingsCookie write function - modify this funtion for new settings
Settings.prototype.writeSettingsCookie = function() {
	settingsCookie.put({
		versionString: this.versionString,
		decryptTestString: this.decryptTestString,
		serversPassphrases: this.serversPassphrases,
		lastSelectedServer: this.lastSelectedServer,
		lastSelectedHash: this.lastSelectedHash
	});
};

// the global settingsCookie read function - modify this funtion for new settings
Settings.prototype.readSettingsCookie = function() {
	var prefs = settingsCookie.get();

	// set to zero version if no cookie available
	if (prefs == null || prefs.versionString == null) {
		settingsCookie.put({
			versionString: "0.0.0"
		});
		// re-read to make prefs object valid
		prefs = settingsCookie.get();
	}

	if (prefs.versionString == this.versionString) {
		if (!prefs.decryptTestString) {
			Mojo.Log.error("decryptTestString not found");
		} else {
			this.decryptTestString = prefs.decryptTestString;
		}

		if (!prefs.serversPassphrases) {
			Mojo.Log.error("serversPassphrases not found");
		} else {
			this.serversPassphrases = prefs.serversPassphrases;
		}

		if (!prefs.lastSelectedServer) {
			Mojo.Log.error("lastSelectedServer not found");
		} else {
			this.lastSelectedServer = prefs.lastSelectedServer;
		}

		if (!prefs.lastSelectedHash) {
			Mojo.Log.error("lastSelectedHash not found");
		} else {
			this.lastSelectedHash = prefs.lastSelectedHash;
		}
	} else {
		var data = {
			version: prefs.versionString
		};
		Mojo.Log.info($L("No import filter available for Cookie version: #{version}. Loading default settings.").interpolate(data));
	}
};

// the global init function - modify this funtion for new settings
Settings.prototype.init = function() {
	this.versionString = "1.0";
	this.decryptTestString = this.encrypt('decryptTestString');
	this.serversPassphrases = [{
		label: 'Configure server(s)',
		value: Base64.encode('demoserver') + "," + this.encrypt('demopassword')
	}];
	this.lastSelectedServer = Base64.encode('demoserver') + "," + this.encrypt('demopassword');
	this.lastSelectedHash = "md5";
};

// Add a get*/set* function for each new value
Settings.prototype.setDecryptTestString = function(decryptTestString) {
	this.decryptTestString = decryptTestString;
	this.writeSettingsCookie();
};

Settings.prototype.getDecryptTestString = function() {
	return this.decryptTestString;
};

Settings.prototype.setServerList = function(serversPassphrases) {
	this.serversPassphrases = serversPassphrases;
	this.writeSettingsCookie();
};

Settings.prototype.getServerList = function() {
	return this.serversPassphrases;
};


Settings.prototype.setLastSelectedServer = function(lastSelectedServer) {
	this.lastSelectedServer = lastSelectedServer;
	this.writeSettingsCookie();
};

Settings.prototype.getLastSelectedServer = function() {
	return this.lastSelectedServer;
};

Settings.prototype.setLastSelectedHash = function(lastSelectedHash) {
	this.lastSelectedHash = lastSelectedHash;
	this.writeSettingsCookie();
};

Settings.prototype.getLastSelectedHash = function() {
	return this.lastSelectedHash;
};


// Crypto functions (encrypt / decrypt) - use Blowfish algorithms provided my Mojo.Model
Settings.prototype.generateKey = function(key) {
	return hex_sha1(key);
};

Settings.prototype.setKey = function(key) {
	this.key = this.generateKey(key);
};

Settings.prototype.getKey = function() {
	return this.key;
};

Settings.prototype.encrypt = function(val) {
	if (this.key != null) {
		return Mojo.Model.encrypt(this.key, val);
	} else {
		return Mojo.Model.encrypt(this.generateKey(""), val);
	}
};

Settings.prototype.encryptWithKey = function(key, val) {
	return Mojo.Model.encrypt(key, val);
};

Settings.prototype.decrypt = function(val) {
	if (this.key != null) {
		return Mojo.Model.decrypt(this.key, val);
	} else {
		return Mojo.Model.decrypt(this.generateKey(""), val);
	}
};

Settings.prototype.decryptWithKey = function(key, val) {
	return Mojo.Model.decrypt(key, val);
};