function foldHash(input, hash) {
	// reduce the hash to 64 bits
	if ((hash == "md4") || (hash == "md5")) {
		return Array(input[0] ^ input[2], input[1] ^ input[3]);
	} else if (hash == "sha1") {
		input = invert_endian(input, true);
		return Array(input[0] ^ input[2] ^ input[4], input[1] ^ input[3]);
	}
}

function computeMD4OTP(otp, sequence) {
	otp = foldHash(core_md4(str2binl(otp), otp.length * 8), "md4");
	for (; sequence > 0; --sequence) {
		otp = foldHash(core_md4(otp, 64), "md4");
	}
	return otp;
}

function computeMD5OTP(otp, sequence) {
	otp = foldHash(core_md5(str2binl(otp), otp.length * 8), "md5");
	for (; sequence > 0; --sequence) {
		otp = foldHash(core_md5(otp, 64), "md5");
	}
	return otp;
}

function computeSHA1OTP(otp, sequence) {
	otp = foldHash(core_sha1(str2binb(otp), otp.length * 8), "sha1");
	for (; sequence > 0; --sequence) {
		otp = foldHash(core_sha1(otp, 64), "sha1");
	}
	otp = invert_endian(otp, true);
	return otp;
}

function otpTo6Word(otp) {
	var parity = 0;
	for (var i = 0; i < 2; ++i) {
		for (var j = 0; j < 32; j += 2) {
			parity += (otp[i] >> j) & 0x3;
		}
	}

	var ind = (otp[0] & 0xff) << 3;
	ind |= (otp[0] >> 13) & 0x7;
	var resOTP = WORDS[ind] + " ";
	ind = ((otp[0] >> 8) & 0x1f) << 6;
	ind |= (otp[0] >> 18) & 0x3f;
	resOTP += WORDS[ind] + " ";
	ind = ((otp[0] >> 16) & 0x3) << 9;
	ind |= ((otp[0] >> 24) & 0xff) << 1;
	ind |= (otp[1] >> 7) & 0x1;
	resOTP += WORDS[ind] + " ";
	ind = (otp[1] & 0x7f) << 4;
	ind |= (otp[1] >> 12) & 0xf;
	resOTP += WORDS[ind] + " ";
	ind = ((otp[1] >> 8) & 0xf) << 7;
	ind |= (otp[1] >> 17) & 0x7f;
	resOTP += WORDS[ind] + " ";
	ind = ((otp[1] >> 16) & 0x1) << 10;
	ind |= ((otp[1] >> 24) & 0xff) << 2;
	ind |= (parity & 0x03);
	resOTP += WORDS[ind];

	return resOTP;
}
